clc
clear
lbl1=['OLS_example_MC1'];
 
logfile_name=strcat(lbl1,'-Diary.log');
delete(logfile_name);
diary(logfile_name);
diary on
fprintf('Hi there. I hope it is a beautiful day in Naperville\n\n');
fprintf('Todays date is: \n'); fprintf(date);
diary off;

should_print=true;
n=200;
k=10;

nrep=400;

bhat=zeros(nrep,k);
shat1=zeros(nrep,k);
shat2=zeros(nrep,k);
shat3=zeros(nrep,k);
shat4=zeros(nrep,k);
fcts=zeros(nrep,1);


for irep=1:nrep;
    
    diary on;
    fprintf('\n')
    fprintf('\n*************************************************\n')
    fprintf('\nReplication Number: %d\n',irep)
    fprintf('\n*************************************************\n')
    diary off;
    
    tic
    rng(7509+31*irep,'twister');
    n=200;
    k=10;

    x=(randn(n,k-1)+randn(n,1)*ones(1,k-1))./sqrt(2);
    x(:,1:3)=(x(:,1:3) >= 0.0);
    x(:,4:6)=x(:,4:6)+1;
    x=[x ones(n,1)];
    x(:,9)=x(:,9)*10;
    x(:,8)=x(:,8)*0.5;
    beta=zeros(k,1);
    beta(k)=5;
    beta(1:5)=(1:5)./5;
    epsilon=randn(n,1).*(1+x(:,1));
    y=x*beta+epsilon;

    [bols,bint,resid,rint,stats] = regress(y,x);
    xe=x.*repmat(resid,1,k);
    vhat=(x'*x)^(-1)*(xe'*xe)*(x'*x)^(-1);
    shat1(irep,:)=sqrt(diag(vhat)');
    bhat(irep,:)=bols';

    dat=[y,x];


    d1=eye(k);
    d2=zeros((k*(k-1)),k);
    ij=1;
    for i=1:(k-1);
        for j=(i+1):k;
            d2(ij,i)=1;
            d2(ij,j)=1;
            ij=ij+1;
            d2(ij,i)=1;
            d2(ij,j)=-1;
            ij=ij+1;
        end;
    end;
    d2=d2';

    nboot=400;
    bols_boot=zeros(nboot,k);
    b_univariate=zeros(nboot,k);
    b_directional=zeros(nboot,(k*(k-1)));

    for iboot=1:nboot
        % Draw a bootstrap sample

        ii=ceil(n*rand(n,1));
        datb=dat(ii,:);
        yb=datb(:,1);
        xb=datb(:,2:k+1);
        resid_OLS=yb-xb*bols;

        % Calculate OLS
        bb=regress(yb,xb);
        bols_boot(iboot,:)=bb';

        % Calculate univarate estimators

        for i=1:size(d1,2)
            bb=regress(resid_OLS,xb(:,i));
            b_univariate(iboot,i)=bb;
        end;

        for i=1:size(d2,2)
            bb=regress(resid_OLS,xb*d2(:,i));
            b_directional(iboot,i)=bb;
        end;


    end


    Vb_Boot=cov(bols_boot);
    OMEGA1=cov(b_univariate);
    OMEGA2=cov([b_univariate b_directional]);

    D2=[d1 d2];
    
    
    [Vcf Hcf V H fct]=solve_for_V_and_H_Version3(OMEGA1, OMEGA2, D2,should_print) ;
    fcts(irep,1)=fct;
    diary on;
    fprintf('\nFunction Value: %8.3f\n',fct);
    diary off;

    Vhat_Alternative=H^(-1)*V*H^(-1);
    Vhat_Alternative_cf=Hcf^(-1)*Vcf*Hcf^(-1);
    shat2(irep,:)=sqrt(diag(Vb_Boot)');
    shat3(irep,:)=sqrt(diag(Vhat_Alternative)');
    shat4(irep,:)=sqrt(diag(Vhat_Alternative_cf)');
    bhats(irep,:)= bols';

    
    
end

tstat1=(bhat-repmat(beta',nrep,1))./shat1;
tstat2=(bhat-repmat(beta',nrep,1))./shat2;
tstat3=(bhat-repmat(beta',nrep,1))./shat3;
tstat4=(bhat-repmat(beta',nrep,1))./shat4;
filename=[lbl1 '_results.mat']; 
save(filename,'beta', 'bhat', 'shat1', 'shat2', 'shat3','shat4','tstat1', 'tstat2', 'tstat3','tstat4','fcts');

a12=mean(abs(tstat1-tstat2));
a13=mean(abs(tstat1-tstat3));
a14=mean(abs(tstat1-tstat4));
a23=mean(abs(tstat2-tstat3));
a24=mean(abs(tstat2-tstat4));
a34=mean(abs(tstat3-tstat4));

diary on

fprintf('\n')
fprintf('\n')
    fprintf('\n*************************************************\n')
    fprintf(lbl1)
    fprintf('\n*************************************************\n')
    fprintf('\nMean Absolute Differences in T-stats \n\n')
    for i=1:k;
        fprintf('%8.3f   &%8.3f   &%8.3f   &%8.3f   &%8.3f   &%8.3f   \\\\ \n',a12(i),a13(i),a14(i),a23(i),a24(i),a34(i)); 
        
    end;
    fprintf('\n')
    fprintf('\nMean Function Value  &%8.3f\n', mean(fcts));


diary off